/***********************************************************************
 *
 * Copyright (C) 2007 Graeme Gott <gottcode@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 **********************************************************************/

#ifndef BOARD_H
#define BOARD_H

#include <QLabel>
class Maze;
class Theme;
class QMainWindow;

class Board : public QLabel
{
	Q_OBJECT
public:
	Board(QMainWindow* parent);
	~Board();

public slots:
	void newGame();
	void loadGame();
	void saveGame();
	void loadSettings();

protected:
	virtual void keyPressEvent(QKeyEvent* event);
	virtual void resizeEvent(QResizeEvent* event);

private:
	void generate();
	void finish();
	void updateTargetMessage();
	QPixmap render() const;
	QPixmap renderDone() const;

	unsigned int m_seed;
	int m_total_targets;
	Maze* m_maze;
	QPoint m_start;
	QList<QPoint> m_targets;
	QLabel* m_status_message;

	// Player
	QPoint m_player;
	int m_player_angle;
	unsigned int m_controls_up;
	unsigned int m_controls_down;
	unsigned int m_controls_left;
	unsigned int m_controls_right;
	unsigned int m_controls_flag;

	// Theme support
	Theme* m_theme;
};

#endif // BOARD_H
