/***********************************************************************
 *
 * Copyright (C) 2007 Graeme Gott <gottcode@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 **********************************************************************/

#include "window.h"

#include "board.h"
#include "settings.h"

#include <QIcon>
#include <QSettings>
#include <QTimer>
#include <QToolBar>

// ============================================================================

Window::Window()
{
	// Create game object
	m_board = new Board(this);
	setCentralWidget(m_board);
	m_board->setFocus();

	// Create settings window
	m_settings = new Settings(this);
	connect(m_settings, SIGNAL(accepted()), m_board, SLOT(loadSettings()));

	// Create toolbar
	QToolBar* toolbar = new QToolBar(this);
	toolbar->setFloatable(false);
	toolbar->setMovable(false);
	toolbar->setIconSize(QSize(22, 22));
	toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
	toolbar->addAction(QIcon(":/new.png"), tr("New"), m_board, SLOT(newGame()));
	toolbar->addAction(QIcon(":/settings.png"), tr("Settings"), m_settings, SLOT(show()));
	toolbar->addAction(QIcon(":/quit.png"), tr("Quit"), this, SLOT(close()));
	addToolBar(toolbar);
	setContextMenuPolicy(Qt::NoContextMenu);

	// Setup window
	setWindowTitle(tr("CuteMaze"));
	resize(QSettings().value("Size", QSize(448, 448)).toSize());

	// Create auto-save timer
	QTimer *timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), m_board, SLOT(saveGame()));
	timer->start(300000);
}

// ============================================================================

void Window::closeEvent(QCloseEvent* event)
{
	QSettings().setValue("Size", size());
	m_board->saveGame();
	QMainWindow::closeEvent(event);
}

// ============================================================================
